/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.aries.samples.blueprint.helloworld.wab;

import org.apache.aries.samples.blueprint.helloworld.api.HelloWorldService;

import java.io.IOException;
import java.io.PrintWriter;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;

public class HelloWorldServlet extends HttpServlet{
      private ServletContext sc;
      private BundleContext bundleContext;
      private static final String OSGI_SERVICE_PREFIX = "osgi:service/";
      
      /**
     * @see Servlet#init(ServletConfig)
     */
    public void init(ServletConfig config) throws ServletException {
       sc = config.getServletContext();       
    }
        
        /**
     * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse response)
     */
    protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        performTask(request, response);
    }

    /**
     * @see HttpServlet#doPost(HttpServletRequest request, HttpServletResponse response)
     */
    protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        performTask(request, response);
    }
    
    protected void performTask(HttpServletRequest request, HttpServletResponse response){
        try {
            bundleContext = (BundleContext) sc.getAttribute("osgi-bundlecontext");
            PrintWriter pw = response.getWriter();
            if (bundleContext != null) {
                ServiceReference sr = bundleContext.getServiceReference(HelloWorldService.class.getName());
                if (sr != null) {
                    HelloWorldService sm = (HelloWorldService) bundleContext.getService(sr);
                    sm.hello(); 
                    pw.println("Call Hello World service Hello");
                    bundleContext.ungetService(sr);
                }
            } else {
                pw.println("Bundle Context is Null");
            }
          
            //JNDI Lookup via osgi:service
            HelloWorldService sm = (HelloWorldService) getOSGIService(HelloWorldService.class.getName(),null);
            if(sm != null){
                sm.hello(); 
                pw.println("Call Hello World service Hello");
            }else {
                pw.println("Service Lookup is Null");
            }
            
           
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } 
    }
     /**
     * Lookup and return an osgi service
     * 
     * @return Object
     * 
     */
    public static final Object getOSGIService(String serviceName, String filter) {
        
        String name = OSGI_SERVICE_PREFIX + serviceName;
        if (filter != null) {
            name = name + "/" + filter;
        }

        try {
            InitialContext ic = new InitialContext();
            return ic.lookup(name);
        } catch (NamingException e) {            
            e.printStackTrace();
            return null;
        }
    }

}


